// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "mtp_data_types.h"

#include <libmtp.h>
#include <glibmm/i18n.h>
#include <cmath>

// Album Type
album_t::album_t(LIBMTP_album_t* _album)
	: Glib::ObjectBase(typeid(album_t)), Glib::Object(), album(_album)
{
}

album_t::~album_t()
{
	if (album) LIBMTP_destroy_album_t(album);
}

Glib::RefPtr<album_t> album_t::create(LIBMTP_album_t* _album)
{
	if (_album)
		return Glib::RefPtr<album_t>(new album_t(_album));
	else
		return Glib::RefPtr<album_t>(new album_t(LIBMTP_new_album_t()));
}

Glib::ustring album_t::RawName() const
{
	return album->name ? album->name : "";
}

Glib::ustring album_t::Name() const
{
	return album->name ? album->name : _("Unknown Album");
}

Glib::ustring album_t::Artist() const
{
	return album->artist ? album->artist : _("Unknown Artist");
}

Glib::ustring album_t::RawArtist() const
{
	return album->artist ? album->artist : "";
}

Glib::ustring album_t::Genre() const
{
	return album->genre ? album->genre : "";
}

uint32_t album_t::Id() const
{
	return album->album_id;
}

uint32_t album_t::NumTracks() const
{
	return album->no_tracks;
}

std::vector<uint32_t> album_t::Tracks() const
{
	return std::vector<uint32_t>(album->tracks,
			album->tracks + album->no_tracks);
}

void album_t::SetName(const Glib::ustring& name)
{
	if (name.empty()) return;
	g_free(album->name);
	album->name = g_strdup(name.c_str());
}

void album_t::SetArtist(const Glib::ustring& artist)
{
	if (artist.empty()) return;
	g_free(album->artist);
	album->artist = g_strdup(artist.c_str());
}

void album_t::SetGenre(const Glib::ustring& genre)
{
	if (genre.empty()) return;
	g_free(album->genre);
	album->genre = g_strdup(genre.c_str());
}

void album_t::SetTracks(std::vector<uint32_t> tracks)
{
	g_free(album->tracks);
	album->tracks = (uint32_t*)g_memdup(&tracks[0], tracks.size() * sizeof(uint32_t)); 
	album->no_tracks = tracks.size();
}

// Track Type
track_t::track_t(LIBMTP_track_t* _track)
	: Glib::ObjectBase(typeid(track_t)), Glib::Object(), track(_track)
{
}

track_t::~track_t()
{
	if (track) LIBMTP_destroy_track_t(track);
}

Glib::RefPtr<track_t> track_t::create(LIBMTP_track_t* _track)
{
	if (_track)
		return Glib::RefPtr<track_t>(new track_t(_track));
	else
		return Glib::RefPtr<track_t>(new track_t(LIBMTP_new_track_t()));
}

Glib::ustring track_t::Name() const
{
	if (track->title)
		return track->title;

	Glib::ustring title = track->filename;
	return title.substr(0, title.find_last_of('.'));
}

uint32_t track_t::Id() const
{
	return track->item_id;
}

uint32_t track_t::Duration() const
{
	return track->duration;
}

Glib::ustring track_t::RawAlbum() const
{
	return track->album ? track->album : "";
}

Glib::ustring track_t::Album() const
{
	return track->album ? track->album : _("Unknown Album");
}

Glib::ustring track_t::RawArtist() const
{
	return track->artist ? track->artist : "";
}

Glib::ustring track_t::Artist() const
{
	return track->artist ? track->artist : _("Unknown Artist");
}

Glib::ustring track_t::Composer() const
{
	return track->composer ? track->composer : "";
}

Glib::ustring track_t::Genre() const
{
	return track->genre ? track->genre : "";
}

Glib::ustring track_t::Date() const
{
	return track->date ? Glib::ustring(track->date).substr(0, 4) : "";
}

uint16_t track_t::Rating() const
{
	return track->rating ? (uint16_t)std::ceil((double)track->rating / (double)20) : 0;
}

uint16_t track_t::TrackNumber() const
{
	return track->tracknumber;
}

uint32_t track_t::UseCount() const
{
	return track->usecount;
}

void track_t::SetFileType(const std::string& filetype)
{
	if (filetype == "audio/mpeg")
		track->filetype = LIBMTP_FILETYPE_MP3;
}

void track_t::SetFileName(const std::string& filename)
{
	if (filename.empty()) return;
	g_free(track->filename);
	track->filename = g_strdup(filename.c_str());
}

void track_t::SetFileSize(const uint64_t size)
{
	track->filesize = size;
}

void track_t::SetTitle(const Glib::ustring& title)
{
	if (title.empty()) return;
	g_free(track->title);
	track->title = g_strdup(title.c_str());
}

void track_t::SetArtist(const Glib::ustring& artist)
{
	if (artist.empty()) return;
	g_free(track->artist);
	track->artist = g_strdup(artist.c_str());
}

void track_t::SetAlbum(const Glib::ustring& album)
{
	if (album.empty()) return;
	g_free(track->album);
	track->album = g_strdup(album.c_str());
}

void track_t::SetGenre(const Glib::ustring& genre)
{
	if (genre.empty()) return;
	g_free(track->genre);
	track->genre = g_strdup(genre.c_str());
}

void track_t::SetDate(const Glib::ustring& date)
{
	if (date.empty()) return;
	g_free(track->date);
	track->date = g_strdup(date.c_str());
}

void track_t::SetTrackNumber(uint16_t track_num)
{
	track->tracknumber = track_num;
}

void track_t::SetNumChannels(uint16_t num_channels)
{
	track->nochannels = num_channels;
}

void track_t::SetDuration(uint32_t duration)
{
	track->duration = duration;
}

// Playlist Type
playlist_t::playlist_t(LIBMTP_playlist_t* _playlist)
	: Glib::ObjectBase(typeid(playlist_t)), Glib::Object(), playlist(_playlist)
{
}

playlist_t::~playlist_t()
{
	if (playlist) LIBMTP_destroy_playlist_t(playlist);
}

Glib::RefPtr<playlist_t> playlist_t::create(LIBMTP_playlist_t* _playlist)
{
	if (_playlist)
		return Glib::RefPtr<playlist_t>(new playlist_t(_playlist));
	else
		return Glib::RefPtr<playlist_t>(new playlist_t(LIBMTP_new_playlist_t()));
}

Glib::ustring playlist_t::Name() const
{
	return playlist->name;
}

uint32_t playlist_t::Id() const
{
	return playlist->playlist_id;
}

uint32_t playlist_t::NumTracks() const
{
	return playlist->no_tracks;
}

std::vector<uint32_t> playlist_t::Tracks() const
{
	return std::vector<uint32_t>(playlist->tracks,
			playlist->tracks + playlist->no_tracks);
}

void playlist_t::SetName(const Glib::ustring& name)
{
	if (name.empty()) return;
	g_free(playlist->name);
	playlist->name = g_strdup(name.c_str());
}

void playlist_t::SetTracks(std::vector<uint32_t> tracks)
{
	g_free(playlist->tracks);
	playlist->tracks = (uint32_t*)g_memdup(&tracks[0], tracks.size() * sizeof(uint32_t)); 
	playlist->no_tracks = tracks.size();
}

albumart_t::albumart_t()
{
	data = LIBMTP_new_filesampledata_t();
	data->filetype = LIBMTP_FILETYPE_JPEG;
}

albumart_t::~albumart_t()
{
	LIBMTP_destroy_filesampledata_t(data);
}

// Sort albums first by name then by artist.
int AlbumCompareNameFirst(const Glib::RefPtr<album_t>& a,
		const Glib::RefPtr<album_t>& b)
{
	int name_compare = a->Name().collate_key().compare(b->Name().collate_key());
	if (name_compare != 0) return name_compare;
	return a->Artist().collate_key().compare(b->Artist().collate_key());
}

// Sort albums first by artist then by name.
int AlbumCompareArtistFirst(const Glib::RefPtr<album_t>& a,
		const Glib::RefPtr<album_t>& b)
{
	int artist_compare = a->Artist().collate_key().compare(b->Artist().collate_key());
	if (artist_compare != 0) return artist_compare;
	return a->Name().collate_key().compare(b->Name().collate_key());
}

// Tracks are sorted first by track number, then by title or filename.  A
// tracknumber of 0 is the greatest.
int TrackCompare(const Glib::RefPtr<const track_t>& a,
		const Glib::RefPtr<const track_t>& b)
{
	if ((a->TrackNumber() == 0 && b->TrackNumber() == 0) ||
			(a->TrackNumber() == b->TrackNumber()))
		return a->Name().collate_key().compare(b->Name().collate_key());
	if (b->TrackNumber() == 0 || a->TrackNumber() < b->TrackNumber())
		return -1;
	return 1;
}

// Returns true if a < b.  Playlists are sorted by name.
bool PlaylistIsLess(const Glib::RefPtr<const playlist_t>& a,
		const Glib::RefPtr<const playlist_t>& b)
{
	return a->Name().collate_key() < b->Name().collate_key();
}
